@@ -6,7 +6,7 @@ from os.path import abspath, dirname, join
 from .listener import TestabilityListener
 from .javascript import JS_LOOKUP
 from .logger import get_logger, argstr, kwargstr
-from .types import WebElementType, LocatorType, OptionalBoolType, OptionalStrType, BrowserLogsType, OptionalDictType, is_firefox
+from .types import WebElementType, LocatorType, OptionalBoolType, OptionalStrType, BrowserLogsType, OptionalDictType, is_firefox, StringArray, StorageType
 from robot.utils import is_truthy, timestr_to_secs, secs_to_timestr
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.common.exceptions import TimeoutException, WebDriverException
@@ -35,8 +35,6 @@ class SeleniumTestability(LibraryComponent):
 
     On top of this, there are some more or less useful utilities for web application testing.
 
-    TODO: Document constructor variables and Listener
-
     == Usage ==
 
     Example:
@@ -51,6 +49,25 @@ class SeleniumTestability(LibraryComponent):
     |   ${x}=   `Testability Loaded`
     |   Should Be True    ${x}
 
+    == Parameters ==
+
+    Plugin can take parameters when it is initialized. All of these values can be modified at runtim too with corresponding keywords. Here's a list of the parameters:
+    ===  automatic_wait ===
+    a truthy value, if SeleniumTestabily should automatically wait for sut to be in state that it can accept more actions.
+    Can be enabled/disable at runtime.
+    Defaults to True
+    === timeout  ===
+    Robot timestring, amount of time to wait for SUT to be in state that it can be safely interacted with.
+    Can be set at runtime.
+    Defaults to 30 seconds.
+    === error_on_timeout ===
+    A truthy value, if timeout does occur either in manual or automatic mode, this determines if error should be thrown that marks marks the exection as failure.
+    Can be enabled/disabled at runtime.
+    Defaults to True
+    === automatic_injection ===
+    A truthy value. User can choose if he wants to instrument the SUT manually with appropriate keywords (or even, if the SUT is instrumented at build time?) or should SeleniumTestability determinine if SUT has testability features and if not then inject & instrument it automatically.
+    Can be enabled/disabled at runtime.
+    Defaults to True
 
     ==  Waiting ==
 
@@ -156,11 +173,6 @@ class SeleniumTestability(LibraryComponent):
         automatic_injection: bool = True,
     ) -> None:
         LibraryComponent.__init__(self, ctx)
-        try:
-            self.ctx.__doc__ = "{}\n\n= SeleniumTestability =\n{}".format(self.ctx.__doc__, self.__doc__)
-        except AttributeError:
-            # plugin will get initialized on every import and at some point __doc__ becomes read-only
-            pass
         self.logger = get_logger("SeleniumTestability")
         self.logger.debug("__init__({},{},{},{},{})".format(ctx, automatic_wait, timeout, error_on_timeout, automatic_injection))
         self.el = ElementKeywords(ctx)
@@ -258,7 +270,7 @@ class SeleniumTestability(LibraryComponent):
             local_error_on_timeout = is_truthy(error_on_timeout)
 
         try:
-            WebDriverWait(self.ctx.driver, local_timeout, 0.15).until(
+            WebDriverWait(self.ctx.driver, local_timeout, 0.15, ignored_exceptions=[TimeoutException]).until(
                 lambda x: self.ctx.driver.execute_async_script(JS_LOOKUP["wait_for_testability"])
             )
         except TimeoutException:
@@ -369,11 +381,75 @@ class SeleniumTestability(LibraryComponent):
 
     @log_wrapper
     @keyword
-    def get_current_useragent(self: "SeleniumTestability") -> str:
+    def get_navigator_useragent(self: "SeleniumTestability") -> str:
         """
         Returns useragent string of current browser.
         """
-        return self.ctx.driver.execute_script(JS_LOOKUP["useragent"])
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "userAgent")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_appCodeName(self: "SeleniumTestability") -> str:
+        """
+        Returns appCoedName string of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "appCodeName")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_appname(self: "SeleniumTestability") -> str:
+        """
+        Returns appName string of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "appName")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_appversion(self: "SeleniumTestability") -> str:
+        """
+        Returns appVersion string of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "appVersion")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_cookieenabled(self: "SeleniumTestability") -> bool:
+        """
+        Returns cookieEnabled boolean of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "cookieEnabled")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_language(self: "SeleniumTestability") -> str:
+        """
+        Returns language string of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "language")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_online(self: "SeleniumTestability") -> bool:
+        """
+        Returns online boolean of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "onLine")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_platform(self: "SeleniumTestability") -> str:
+        """
+        Returns platform string of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "platform")
+
+    @log_wrapper
+    @keyword
+    def get_navigator_product(self: "SeleniumTestability") -> str:
+        """
+        Returns product string of current browser.
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["navigator"], "product")
 
     @log_wrapper
     @keyword
@@ -490,7 +566,9 @@ class SeleniumTestability(LibraryComponent):
     def element_should_be_blocked(self: "SeleniumTestability", locator: LocatorType) -> None:
         """
         Throws exception if element found with ``locator`` is not blocked by any overlays.
-        #TODO: Add examples
+        Example:
+        | Element Should Be Blocked  |  id:some_id |
+        If nothing is on top of of provided element, throws an exception
         """
         is_blocked = self._element_blocked(locator)
         if not is_blocked:
@@ -501,14 +579,16 @@ class SeleniumTestability(LibraryComponent):
     def element_should_not_be_blocked(self: "SeleniumTestability", locator: LocatorType) -> None:
         """
         Throws exception if element found with ``locator`` is being blocked by overlays.
-        #TODO: Add examples
+        Example:
+        | Element Should Not Be Blocked  |  id:some_id |
+        If there's element on top of provided selector, throws an exception
         """
         is_blocked = self._element_blocked(locator)
         if is_blocked:
             raise AssertionError("Element with locator {} is blocked".format(locator))
 
     def _get_ff_log(self: "SeleniumTestability", name: str) -> BrowserLogsType:
-        matcher = r"^(?P<source>JavaScript|console)(\s|\.)(?P<level>warn.*?|debug|trace|log|error|info):\s(?P<message>.*)$"
+        matcher = r"^(?P<source>JavaScript|console)(\s|\.)(?P<level>warn.*?|debug|trace|log|error|info):\s(?P<message>(?!resource:).*)$"
         LEVEL_LOOKUP = {
             "log": "INFO",
             "warn": "WARN",
@@ -521,7 +601,7 @@ class SeleniumTestability(LibraryComponent):
         SOURCE_LOOKUP = {"JavaScript": "javascript", "console": "console-api"}
         log = []
         skip_lines = self.ff_log_pos.get(name, 0)
-        buff = []
+        buff: BrowserLogsType = []
         with open(name, "r") as f:
             buff = f.read().split("\n")
         self.ff_log_pos[name] = skip_lines + len(buff)
@@ -587,6 +667,70 @@ class SeleniumTestability(LibraryComponent):
         from_element = self.el.find_element(locator)
         self.ctx.driver.execute_script(JS_LOOKUP["set_element_attribute"], from_element, attribute, value)
 
+    @log_wrapper
+    @keyword
+    def get_location_hash(self: "SeleniumTestability") -> str:
+        """
+        returns the fragment identifier of the URL prefexed by a '#'
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "hash")
+
+    @log_wrapper
+    @keyword
+    def get_location_host(self: "SeleniumTestability") -> str:
+        """
+        returns the hostname and the port of the URL appended by a ':'
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "host")
+
+    @log_wrapper
+    @keyword
+    def get_location_hostname(self: "SeleniumTestability") -> str:
+        """
+        return the hostname of the URL
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "hostname")
+
+    @log_wrapper
+    @keyword
+    def get_location_href(self: "SeleniumTestability") -> str:
+        """
+        returns the entire URL
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "href")
+
+    @log_wrapper
+    @keyword
+    def get_location_origin(self: "SeleniumTestability") -> str:
+        """
+        returns the canonical form of the origin of the specific location
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "origin")
+
+    @log_wrapper
+    @keyword
+    def get_location_port(self: "SeleniumTestability") -> str:
+        """
+        returns the port number of the URL
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "port")
+
+    @log_wrapper
+    @keyword
+    def get_location_protocol(self: "SeleniumTestability") -> str:
+        """
+        returns the protocol scheme of the URL
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "protocol")
+
+    @log_wrapper
+    @keyword
+    def get_location_search(self: "SeleniumTestability") -> str:
+        """
+        returns the sting containing a '?' followed by the parameters or ``querystring`` of the URL
+        """
+        return self.ctx.driver.execute_script(JS_LOOKUP["get_window_location"], "search")
+
     @log_wrapper
     @keyword
     def generate_firefox_profile(
@@ -610,3 +754,40 @@ class SeleniumTestability(LibraryComponent):
 
         profile.update_preferences()
         return profile
+
+    @log_wrapper
+    @keyword
+    def get_storage_length(self: "SeleniumTestability", storage_type: str = "localStorage") -> int:
+        return self.ctx.driver.execute_script(JS_LOOKUP["storage_length"], storage_type)
+
+    @log_wrapper
+    @keyword
+    def get_storage_keys(self: "SeleniumTestability", storage_type: str = "localStorage") -> StringArray:
+        return self.ctx.driver.execute_script(JS_LOOKUP["storage_keys"], storage_type)
+
+    @log_wrapper
+    @keyword
+    def get_storage_item(self: "SeleniumTestability", key: str, storage_type: str = "localStorage") -> StorageType:
+        matcher = r"^{.*}$"
+        storage_item = self.ctx.driver.execute_script(JS_LOOKUP["storage_getitem"], storage_type, key)
+        print(f"TYPE OF {key} is {type(storage_item)}")
+        if isinstance(storage_item, str) and re.match(matcher, storage_item):
+            storage_item = json.loads(storage_item)
+        return storage_item
+
+    @log_wrapper
+    @keyword
+    def set_storage_item(self: "SeleniumTestability", key: str, value: StorageType, storage_type: str = "localStorage") -> None:
+        if isinstance(value, dict):
+            value = json.dumps(value)
+        self.ctx.driver.execute_script(JS_LOOKUP["storage_setitem"], storage_type, key, value)
+
+    @log_wrapper
+    @keyword
+    def clear_storage(self: "SeleniumTestability", storage_type: str = "localStorage") -> None:
+        self.ctx.driver.execute_script(JS_LOOKUP["storage_clear"], storage_type)
+
+    @log_wrapper
+    @keyword
+    def remove_storage_item(self: "SeleniumTestability", key: str, storage_type: str = "localStorage") -> None:
+        return self.ctx.driver.execute_script(JS_LOOKUP["storage_removeitem"], storage_type, key)
